#include "stdafx.h"
#include "Sys\timeb.h"
#include <io.h>
#include <Math.h>

//=================================================== Registry Interface 
CString g_cRegPath = "Software\\UtilityDefaultPath";

void RegistrySetPath( const char * pchPath ) {
	g_cRegPath.Set(pchPath);
	}
	
CString RegistryLoadString( const char * pchKey ) { 
	CString cResult;

	//-- Open Reg Key 
	HKEY h;
	if( ERROR_SUCCESS != RegCreateKeyEx( HKEY_CURRENT_USER, g_cRegPath.ConstBuffer(), 0, NULL, 0, KEY_ALL_ACCESS, NULL, &h, NULL ) ) 
		return "";

	//-- Query string length
	DWORD dwSize=0;
	if( ERROR_SUCCESS != RegQueryValueEx( h, pchKey, 0, NULL, NULL, &dwSize ) ) 
		return "";

	//-- allocate buffer , get string 
	char * buf = new char[dwSize];
	if( ERROR_SUCCESS == RegQueryValueEx( h, pchKey, 0, NULL, (LPBYTE)buf, &dwSize ) ) 
		cResult = buf;
	RegCloseKey( h );
	delete [] buf;

	return cResult;
	}

bool RegistrySaveString( const char * pchKey, const char * pchValue ) { 
	bool bResult = true;
	HKEY h;
	if( ERROR_SUCCESS != RegCreateKeyEx( HKEY_CURRENT_USER, g_cRegPath.ConstBuffer(), 0, NULL, 0, KEY_ALL_ACCESS, NULL, &h, NULL ) ) 
		return false;
	if( ERROR_SUCCESS != RegSetValueEx( h, pchKey, 0, REG_SZ, (BYTE *)pchValue, strlen(pchValue)+1 ) ) 
		bResult=false;
	RegCloseKey( h );
	return bResult;
	}
	
//-----	
CRegistryString :: CRegistryString( const char * pchKey ){ 
	m_cKey = pchKey;
	}

bool CRegistryString :: Save( ){ 
	bool bResult = true;
	HKEY h;
	if( ERROR_SUCCESS != RegCreateKeyEx( HKEY_CURRENT_USER, g_cRegPath.ConstBuffer(), 0, NULL, 0, KEY_ALL_ACCESS, NULL, &h, NULL ) ) 
		return false;
	if( ERROR_SUCCESS != RegSetValueEx( h, m_cKey.ConstBuffer(), 0, REG_SZ, (BYTE *)m_cValue.ConstBuffer(), m_cValue.Length()+1 ) ) 
		bResult=false;
	RegCloseKey( h );
	return bResult;
	}

bool CRegistryString :: Load( ){ 
	//-- Open Reg Key 
	HKEY h;
	if( ERROR_SUCCESS != RegCreateKeyEx( HKEY_CURRENT_USER, g_cRegPath.ConstBuffer(), 0, NULL, 0, KEY_ALL_ACCESS, NULL, &h, NULL ) ) 
		return false;

	//-- Query string length
	DWORD dwSize=0;
	if( ERROR_SUCCESS != RegQueryValueEx( h, m_cKey.ConstBuffer(), 0, NULL, NULL, &dwSize ) ) 
		return false;

	//-- allocate buffer , get string 
	CString cResult;

	char * buf = new char[dwSize];
	long iResult = RegQueryValueEx( h, m_cKey.ConstBuffer(), 0, NULL, (LPBYTE)buf, &dwSize );
	RegCloseKey( h );
	
	if( ERROR_SUCCESS != iResult ) {
		delete [] buf;
		return false;
		}
		
	m_cValue = CString( buf, dwSize );
	int iLen = m_cValue.Length();
	if( m_cValue[iLen-1]=='\0' ) 
		m_cValue=m_cValue.Left( iLen-1 );
	delete [] buf;

	return true;
	}

void CRegistryString :: AppendInt( int i ){ 
	CString cI( i );
	CString cILen( cI.Length(), "%2d" );
	m_cValue += cILen;
	m_cValue += cI;
	}

void CRegistryString :: AppendDouble( double d ){ 
	CString cD( d );
	CString cDLen( cD.Length(), "%3d" );
	m_cValue += cDLen;
	m_cValue += cD;
	}

void CRegistryString :: AppendString( const char * pch ){ 
	CString cLen( int(strlen(pch)), "%4d" );
	m_cValue += cLen;
	m_cValue += pch;
	}

void CRegistryString :: AppendBool( bool b ){
	m_cValue.Append( b ? 'Y' : 'N' );
	}

int CRegistryString :: ExtractInt( ){ 
	int iLen = atoi(m_cValue.Left(2).ConstBuffer());
	int iVal = atoi(m_cValue.Mid(2,iLen).ConstBuffer());
	m_cValue=m_cValue.Mid(2+iLen);
	return iVal;
	}

double CRegistryString :: ExtractDouble( ){ 
	int iLen = atoi(m_cValue.Left(3).ConstBuffer());
	double dVal = atof(m_cValue.Mid(3,iLen).ConstBuffer());
	m_cValue=m_cValue.Mid(3+iLen);
	return dVal;
	}

CString CRegistryString :: ExtractString( ){ 
	int iLen = atoi(m_cValue.Left(4).ConstBuffer());
	CString cValue = m_cValue.Mid(4,iLen);
	m_cValue=m_cValue.Mid(4+iLen);
	return cValue;
	}

bool CRegistryString :: ExtractBool(){
	bool b = m_cValue[0]=='Y';
	m_cValue=m_cValue.Mid(1);
	return b;
	}
		
//-----------------------------------------------------------------------------
double atof_l( const char * pchBuf, int iLen ) { 
	char * p = new char[iLen+1];
	memcpy( p, pchBuf, iLen );
	p[iLen]=0;
	double dResult=atof(p);
	delete [] p;
	return dResult;
	}

int atoi_l( const char * pchBuf, int iLen ) { 
	char * p = new char[iLen+1];
	memcpy( p, pchBuf, iLen );
	p[iLen]=0;
	int iResult=atoi(p);
	delete [] p;
	return iResult;
	}

//-----------------------------------------------------------------------------
bool CBuffer :: Allocate( int iSize ) { 
	if( iSize < m_iSize ) 
		return true;
	char * p = new char[iSize+JUMPSIZE];
	if( p == NULL ) 
		return false;
	if( m_btBuffer != NULL ) {
		memcpy( p, m_btBuffer, m_iLength );
		delete [] m_btBuffer;
		}
	m_btBuffer = p;
	m_iSize=iSize+JUMPSIZE;
	return true;
	}
	
CBuffer :: CBuffer( int iInitSize ) { 
	m_btBuffer=NULL;
	m_iPos=0;
	m_iSize=0;
	m_iLength=0;
	Allocate( iInitSize );
	}
	
bool CBuffer :: AppendInt( int iData ) { 
	char buf1[16];
	char buf2[3];
	sprintf( buf1, "%d", iData );
	int iLen = strlen(buf1);
	sprintf( buf2, "%2d", iLen);
	if( ! Allocate( m_iLength+3+iLen ) )
		return false;
	memcpy( m_btBuffer+m_iLength, buf2, 2 );
	memcpy( m_btBuffer+m_iLength+2, buf1, iLen );
	m_iLength += 2+iLen;
	return true;
	}
	
bool CBuffer :: AppendString( const char * pchData ) { 
	char buf[5];
	int iLen = min(9999,strlen( pchData ));
	sprintf( buf, "%4d", iLen );
	if( ! Allocate( m_iLength+5+iLen ) ) 
		return false;
	memcpy( m_btBuffer+m_iLength, buf, 4 );
	memcpy( m_btBuffer+m_iLength+4, pchData, iLen );
	m_iLength += 4+iLen;
	return true;
	}
	
bool CBuffer :: AppendDouble( double d ) { 
	char buf1[256];
	char buf2[3];
	sprintf( buf1, "%f", d );
	int iLen = min( 99, strlen(buf1));
	sprintf( buf2, "%2d", iLen);
	if( ! Allocate( m_iLength+3+iLen ) )
		return false;
	memcpy( m_btBuffer+m_iLength, buf2, 2 );
	memcpy( m_btBuffer+m_iLength+2, buf1, iLen );
	m_iLength += 2+iLen;
	return true;
	}

void CBuffer :: Fill( int iLen, const char * pBuf ){
	if( ! Allocate( iLen ) ) 
		return ;
	memcpy( m_btBuffer, pBuf, iLen );
	m_iLength=iLen;
	m_iPos=0;
	}
	
bool CBuffer :: ExtractInt( int * pi ){
	int iLen = atoi_l( m_btBuffer+m_iPos, 2 );
	m_iPos+=2;
	if( iLen<=0 || iLen>16 || m_iPos+iLen>m_iLength ) 
		return false;
	*pi = atoi_l( m_btBuffer+m_iPos, iLen );
	m_iPos+=iLen;
	return true;	
	}
	
bool CBuffer :: ExtractString( CString * pc ){
	int iLen=atoi_l( m_btBuffer+m_iPos, 4 );
	m_iPos+=4;
	if( iLen<=0 || iLen>9999 )
		return false;
	pc->Set( m_btBuffer+m_iPos, iLen );
	m_iPos += iLen;
	return true;
	}
	
bool CBuffer :: ExtractDouble( double * pd ){
	int iLen=atoi_l( m_btBuffer+m_iPos, 2 );
	m_iPos+=2;
	if( iLen<=0 || iLen>100 || m_iPos+iLen>m_iLength ) 
		return false;
	*pd = atof_l( m_btBuffer+m_iPos, iLen );
	m_iPos+=iLen;
	return true;	
	}

CString CBuffer :: ToString() { 
	Allocate( m_iLength+1 );
	m_btBuffer[m_iLength]=0;
	return CString( m_btBuffer );
	}

//-----------------------------------------------------------------------------
bool FileExists( const char * pchFile ) { 
	return _access( pchFile, 0 ) == 0;
	}

bool FileLoadString( FILE * h, CString * pC ) {
	int iLen;
	if( fread( &iLen, sizeof(int), 1, h ) != 1 ) {
		fseek( h, iLen, SEEK_CUR );
		return false;
		}
	if( iLen<0 ) 
		return false;
	pC->allocate( iLen+1 );
	if( fread( pC->Buffer(), 1, iLen, h ) != iLen ) {
		return false;
		}
	pC->setlength();
	return true;
	}

bool FileLoadString( FILE * h, char * pch, int iMax ) {
	int iLen;
	if( fread( &iLen, sizeof(int), 1, h ) != 1 || iLen<0 || iLen>=iMax ) {
		fseek( h, iLen, SEEK_CUR );
		return false;
		}
	pch[iLen]=0;
	if( fread( pch, 1, iLen, h ) != iLen ) 
		return false;
	return true;
	}

//-----------------------------------------------------------------------------
bool FileSaveString( FILE * h, char * pch ) {
	int iLen = 1+(int)strlen(pch);
	if( fwrite( &iLen, sizeof(int), 1, h ) != 1 ) 
		return false;
	if( fwrite( pch, iLen, 1, h ) != 1 ) 
		return false;
	return true;
	}

int FileStringLength( char * pch ) { 
	return sizeof(int)+strlen(pch)+1;
	}

//-----------------------------------------------------------------------------
CString :: CString() { 
	init();
	m_iAlloc=0;
	m_pch[0]=0;
	}

CString :: ~CString() { 
	if( m_pch ) 
		delete [] m_pch;
	}

CString :: CString( char ch ) { 
	m_iAlloc=0;
	init(2);
	m_pch[0]=ch;
	m_pch[1]=0;
	setlength();
	}

CString :: CString( const char * pch ) { 
	m_iAlloc=0;
	init(strlen(pch)+1);
	strcpy( m_pch, pch );
	setlength();
	}

CString :: CString( const char * pch, int iLen ) { 
	m_iAlloc=0;
	init(iLen+1);
	memcpy( m_pch, pch, iLen );
	m_pch[iLen]=0;
	setlength();
	}

CString :: CString( const CString & pC ) { 
	m_iAlloc=0;
	init( pC.Length()+1 );
	strcpy( m_pch, pC.ConstBuffer() );
	setlength();
	}

CString :: CString( int iValue, const char * pchFormat ) {
	m_iAlloc=0;
	char buf[128];
	sprintf( buf, pchFormat, iValue );
	init( strlen(buf)+1 );
	strcpy( m_pch, buf );
	setlength();
	}

CString :: CString( double dValue, const char * pchFormat ) {
	m_iAlloc=0;
	char buf[128];
	sprintf( buf, pchFormat, dValue );
	init( strlen(buf)+1 );
	strcpy( m_pch, buf );
	setlength();
	}

void CString :: Append( const char * pch ) { 
	allocate( m_iLength + strlen(pch) + 1 );
	strcpy( m_pch+m_iLength, pch );
	setlength();
	}

void CString :: Append( const char * pch, int iLen ) { 
	allocate( m_iLength+iLen+1 );
	memcpy( m_pch+m_iLength, pch, iLen );
	m_pch[m_iLength+iLen]=0;
	m_iLength+=iLen;
	}
	
void CString :: Append( char ch ) { 
	allocate( m_iLength + 2 );
	m_pch[m_iLength]=ch;
	m_pch[++m_iLength]=0;
	}

void CString :: Truncate( int iPos ) { 
	if( iPos>=0 && iPos<m_iLength ) { 
		m_pch[iPos]=0;
		setlength();
		}
	}

CString CString :: RTrim( ) {
	int iLen=m_iLength; 
	while( iLen>0 && (m_pch[iLen-1]==' ' || m_pch[iLen-1]=='\t' ) )
		--iLen;
	CString o( ConstBuffer(), iLen );
	return o;
	}
	
int ZTrim( char * pchBuf ) { 
	int iLen=strlen(pchBuf);
	while( iLen>2 && strchr(pchBuf,'.')!=NULL && pchBuf[iLen-1]=='0' && pchBuf[iLen-2]!='.' ) 
		pchBuf[--iLen]=0;
	return iLen;
	}
							

CString CString :: Mid( int iPos, int iLen ) {
	CString o( ConstBuffer()+max(0,min(m_iLength,iPos)) );
	if( iLen >= 0 ) 
		o.Truncate( iLen );
	return o;
	}

CString CString :: Left( int iLen ) {
	return Mid( 0, iLen );
	}

CString CString :: Right( int iLen ) {
	return Mid( max(0,m_iLength-iLen), min(m_iLength,iLen) );
	}
	
void CString :: ToUpper( ) { 
	for( int i=0; i<m_iLength; ++i ) 
		if( m_pch[i]>='a' && m_pch[i]<='z' )
			m_pch[i] -= ('a'-'A');
	}
void CString :: ToLower( ) { 
	for( int i=0; i<m_iLength; ++i ) 
		if( m_pch[i]>='A' && m_pch[i]<='Z' )
			m_pch[i] -= ('A'-'a');
	}


char * CString :: ConstBuffer() const { 
	return m_pch;
	}

char * CString :: Buffer() { 
	return m_pch;
	}

int CString :: Length() const {
	return m_iLength; 
	}

int CString :: TrimLength() const { 
	int iLen = m_iLength;
	while( iLen>0 && ( m_pch[iLen-1]==' ' || m_pch[iLen-1]=='\t' ) ) 
		--iLen;
	return iLen;
	}

void CString :: init(int iLen) { 
	m_pch=new char[iLen];
	m_iLength=0;
	m_iAlloc=( m_pch==NULL ? 0 : iLen );
	}

bool CString :: allocate(int i) { 
	if( m_iAlloc>=i ) 
		return true;
	char * pch = new char[i];
	if( pch == NULL ) 
		return false;
	if( m_pch != NULL ) { 
		memcpy( pch, m_pch, m_iAlloc );
		delete [] m_pch;
		}
	m_pch=pch;
	m_iAlloc=i;
	return true;
	}

void CString :: setlength() { 
	if( m_pch == NULL ) 
		m_iLength=0;
	else 
		m_iLength=strlen(m_pch);
	}

void CString :: Set( const char * pch ) { 
	allocate( strlen(pch)+1 );
	strcpy( m_pch, pch );
	setlength();
	}

void CString :: Set( const char * pch, int iLen ) { 
	allocate( iLen+1 );
	memcpy( m_pch, pch, iLen );
	m_pch[iLen]=0;					// ensure the 0 exists, though it is past end.
	m_iLength=iLen;
	}

CString& CString :: operator += ( const CString & pc ) { 
	Append( pc.ConstBuffer() );
	setlength();
	return *this;
	}

CString& CString :: operator += ( const char * pch ) { 
	Append( pch );
	setlength();
	return *this;
	}
	
bool operator == ( CString & c1, CString & c2 ) { 
	return strcmp( c1.ConstBuffer(), c2.ConstBuffer() ) == 0;
	}
	
bool operator == ( CString & c, const char * pch ) { 
	return strcmp( c.ConstBuffer(), pch ) == 0;
	}
	
bool operator <= ( CString & c1, CString & c2 ) { 
	return strcmp( c1.ConstBuffer(), c2.ConstBuffer() ) <= 0;
	}
	
bool operator <= ( CString & c, const char * pch ) { 
	return strcmp( c.ConstBuffer(), pch ) <= 0;
	}
	
char CString :: operator [ ] (int iIndex){
	if( iIndex<0 || iIndex>=m_iLength ) 
		return 0;
	return m_pch[iIndex];
	}

const CString & CString::operator = ( const CString & pC ) { 
	if( & pC != this ) 
		Set( pC.ConstBuffer() );
	return *this;
	}
		
const CString & CString::operator = ( const char * pch ) { 
	Set( pch );
	return * this;
	}

int CString :: Find( char ch, int iStart ) { 
	if( iStart>=m_iLength ) 
		return -1;
	char * pch = strchr( m_pch+max(0,iStart), ch );
	return pch==NULL?-1:pch-m_pch;
	}
	
int CString :: Find( const char * pch, int iStart ) { 
	int iLen=strlen(pch);
	if( iStart>=m_iLength-iLen ) 
		return -1;
	char * pchFind = strstr( m_pch+max(0,iStart), pch );
	return pchFind==NULL?-1:pchFind-m_pch;
	}

//-----------------------------------------------------------------------------
int CStringArray :: Count(){
	return m_iStrings;
	}
	
CStringArray :: CStringArray(int iStringsPerAlloc){
	m_iStringsPerAlloc=max(1,iStringsPerAlloc);
	m_iStrings=0;
	m_iAllocated=0;
	m_aStrings = new CString[m_iStringsPerAlloc];
	if( m_aStrings!=NULL ) 
		m_iAllocated=m_iStringsPerAlloc;
	}
	
CStringArray :: ~CStringArray(){
	delete [] m_aStrings;
	}
	
bool CStringArray :: AddString( const char * pch, int iPos ){
	//-- expand array if necc.
	if( m_iStrings == m_iAllocated ) { 
		CString * p = new CString [m_iAllocated+m_iStringsPerAlloc];
		if( p == NULL ) 
			return false;
		for( int i=0; i<m_iStrings; ++i ) 
			p[i]=m_aStrings[i];
		delete [] m_aStrings;
		m_aStrings=p;
		m_iAllocated+=m_iStringsPerAlloc;
		}
	//-- add String at specified pos
	if( iPos<0 || iPos>m_iStrings ) 
		iPos=m_iStrings;
	for( int i=m_iStrings; i>iPos; --i ) 
		m_aStrings[i]=m_aStrings[i-1];
	m_aStrings[iPos].Set(pch);
	++m_iStrings;	
	return true;
	}
	
bool CStringArray :: DeleteString( int iPos ){
	if( iPos<0 || iPos>=m_iStrings )
		return false;
	--m_iStrings;
	for( int i=iPos; i<m_iStrings; ++i ) 
		m_aStrings[i]=m_aStrings[i+1];
	return true;
	}
	
CString& CStringArray :: GetAt( int iIndex ){
	if( iIndex>=0 && iIndex<m_iStrings )
		return m_aStrings[iIndex];
	return CString("");
	}
	
CString CStringArray :: operator [ ] ( int iIndex ){
	return GetAt(iIndex);
	}
	
int CStringArray :: Find( const char * pchText ) { 
	for( int i=0; i<m_iStrings; ++i ) 
		if( strcmp(m_aStrings[i].ConstBuffer(),pchText)==0 ) 
			return i;
	return -1;
	}
	
void CStringArray :: Swap(int iPos, int jPos)
{
	CString tmp = m_aStrings[iPos];
	m_aStrings[iPos] = m_aStrings[jPos];
	m_aStrings[jPos] = tmp;
}

/*
bool CStringArray :: MoveString( int iFrom, int iTo ){
	if( iFrom<0 || iFrom>m_iStrings || iTo<0 || iTo>m_iStrings ) 
		return false;
	CString cFrom = GetAt(iFrom);
	int iIncr = (iFrom<iTo) ? 1 : -1;
	for( int iPos=iFrom; iPos!=iTo; iPos+=iIncr )
		m_aStrings[iPos]=m_aStrings[iPos+iIncr];
	m_aStrings[iTo]=cFrom;
	return true;
	}
CString CStringArray :: First( ){
	if( m_iStrings>0 )
		return m_aStrings[0];
	return CString("");
	}
CString CStringArray :: Last( ){
	if( m_iStrings>0 )
		return m_aStrings[m_iStrings-1];
	return CString("");
	}
*/

//-----------------------------------------------------------------------------
bool StringToFile( CString * pc, const char * pchFile ) { 
	FILE * h = fopen( pchFile, "w" );
	if( h==NULL ) 
		return false;
	fwrite( pc->ConstBuffer(), pc->Length(), 1, h );
	fclose( h );
	return true;
	}

bool StringFromFile( const char * pchFile, CString * pc ) { 
	FILE * h = fopen( pchFile, "rb" );
	if( h==NULL )
		return false;

	fseek( h, 0, SEEK_END );
	int iLen = ftell(h);
	fseek( h, 0, SEEK_SET );

	char * ach = new char[iLen];
	if( ach==NULL || fread( ach, iLen, 1, h ) != 1 ) { 
		fclose( h );
		delete [] ach;
		return false;
		}
	pc->Set( ach, iLen );
	fclose( h );
	delete [] ach;
	return true;
	}

//-----------------------------------------------------------------------------
bool StringsToFile( CStringArray * pac, const char * pchFile, int iMax ) { 
	FILE * h = fopen( pchFile, "w" );
	if( h==NULL ) 
		return false;
	for( int i=0; i<pac->Count() && i<iMax; ++i ) 
		fprintf( h, "%s\n", pac->GetAt(i).ConstBuffer() );
	fclose( h );
	return true;
	}
	
bool StringsFromFile( CStringArray * pac, const char * pchFile, int iMax ) { 
	char buf[1000];
	buf[999]=0;
	FILE * h = fopen( pchFile, "r" );
	if( h==NULL ) 
		return false;
		
	int iLine=0, iLen;
	while( fgets( buf, 999, h ) != NULL && ++iLine<iMax ) { 
		while( (iLen=strlen(buf))>0 && buf[iLen-1]<32 ) 
			buf[iLen-1]=0;
		pac->AddString( buf );
		}
	fclose(h);
	return true;
	}

//-----------------------------------------------------------------------------
int CPtrArray :: Count(){
	return m_iPtrs;
	}
CPtrArray :: CPtrArray(int iPtrsPerAlloc){
	m_iPtrsPerAlloc=max(1,iPtrsPerAlloc);
	m_iPtrs=0;
	m_iAllocated=0;
	m_aPtrs = new void *[m_iPtrsPerAlloc];
	if( m_aPtrs!=NULL ) 
		m_iAllocated=m_iPtrsPerAlloc;
	}
CPtrArray :: ~CPtrArray(){
	delete [] m_aPtrs;
	}
bool CPtrArray :: AddPtr( void * pData, int iPos ){
	//-- expand array if necc.
	if( m_iPtrs == m_iAllocated ) { 
		void ** p = new void *[m_iAllocated+m_iPtrsPerAlloc];
		if( p == NULL ) 
			return false;
		for( int i=0; i<m_iPtrs; ++i ) 
			p[i]=m_aPtrs[i];
		delete [] m_aPtrs;
		m_aPtrs=p;
		m_iAllocated+=m_iPtrsPerAlloc;
		}
	//-- add Ptr at specified pos
	if( iPos<0 || iPos>m_iPtrs ) 
		iPos=m_iPtrs;
	for( int i=m_iPtrs; i>iPos; --i ) 
		m_aPtrs[i]=m_aPtrs[i-1];
	m_aPtrs[iPos]=pData;
	++m_iPtrs;	
	return true;
	}
bool CPtrArray :: DeletePtr( int iPos ){
	if( iPos<0 || iPos>=m_iPtrs )
		return false;
	--m_iPtrs;
	for( int i=iPos; i<m_iPtrs; ++i ) 
		m_aPtrs[i]=m_aPtrs[i+1];
	return true;
	}
void * CPtrArray :: GetAt( int iIndex ){
	if( iIndex>=0 && iIndex<m_iPtrs )
		return m_aPtrs[iIndex];
	return NULL;
	}
void * CPtrArray :: operator [ ] ( int iIndex ){
	return GetAt(iIndex);
	}
/*
bool CPtrArray :: MovePtr( int iFrom, int iTo ){
	if( iFrom<0 || iFrom>m_iPtrs || iTo<0 || iTo>m_iPtrs ) 
		return false;
	void * cFrom = GetAt(iFrom);
	int iIncr = (iFrom<iTo) ? 1 : -1;
	for( int iPos=iFrom; iPos!=iTo; iPos+=iIncr )
		m_aPtrs[iPos]=m_aPtrs[iPos+iIncr];
	m_aPtrs[iTo]=cFrom;
	return true;
	}
void * CPtrArray :: First( ){
	if( m_iPtrs>0 )
		return m_aPtrs[0];
	return NULL;
	}
void * CPtrArray :: Last( ){
	if( m_iPtrs>0 )
		return m_aPtrs[m_iPtrs-1];
	return NULL;
	}
*/

//-----------------------------------------------------------------------------
// CItemArray Implementation : Array of pointer to objects descended from CItem.
CItem :: ~CItem(){
	}

CItemArray :: CItemArray(int iItemsPerAlloc){
	m_iItemsPerAlloc=max(0,iItemsPerAlloc);
	m_iItems=0;
	m_iAllocated=max(m_iItemsPerAlloc,25);
	m_aItems = new CItem *[m_iAllocated];
	if( m_aItems==NULL ) 
		m_iAllocated=0;
	}
	
CItemArray :: ~CItemArray(){
	DeleteAll();
	}

int CItemArray :: Count() const { 
	return m_iItems;
	}

void CItemArray :: DeleteAll(){
	for( int i=0; i<Count(); ++i )
		delete GetAt(i);
	m_iItems=0;
	}

bool CItemArray :: AddItem( CItem * poItem, int iPos ){
	//-- expand array if necc.
	if( m_iItems == m_iAllocated ) { 
		int iAlloc = m_iItemsPerAlloc==0 ? m_iAllocated : m_iItemsPerAlloc;
		CItem ** p = new CItem *[m_iAllocated+iAlloc];
		if( p == NULL ) 
			return false;
		memcpy( p, m_aItems, sizeof( CItem *) * m_iItems );
		//for( int i=0; i<m_iItems; ++i ) 
		//	p[i]=m_aItems[i];
		delete [] m_aItems;
		m_aItems=p;
		m_iAllocated+=iAlloc;
		}
		
	//-- add item at specified pos
	if( iPos<0 || iPos>m_iItems ) 
		iPos=m_iItems;
	for( int i=m_iItems; i>iPos; --i ) 
		m_aItems[i]=m_aItems[i-1];
	m_aItems[iPos]=poItem;
	++m_iItems;	
	return true;
	}

bool CItemArray :: DeleteItem(int iPos){
	if( iPos<0 || iPos>=m_iItems )
		return false;
	delete m_aItems[iPos];
	--m_iItems;
	for( int i=iPos; i<m_iItems; ++i ) 
		m_aItems[i]=m_aItems[i+1];
	return true;
	}

bool CItemArray :: MoveItem( int iFrom, int iTo ){
	if( iFrom<0 || iFrom>m_iItems || iTo<0 || iTo>m_iItems ) 
		return false;
	CItem * pFrom = GetAt(iFrom);
	int iIncr = (iFrom<iTo) ? 1 : -1;
	for( int iPos=iFrom; iPos!=iTo; iPos+=iIncr )
		m_aItems[iPos]=m_aItems[iPos+iIncr];
	m_aItems[iTo]=pFrom;
	return true;
	}

CItem * CItemArray :: GetAt( int iIndex ) const {
	if( iIndex>=0 && iIndex<m_iItems )
		return m_aItems[iIndex];
	return NULL;
	}

CItem * CItemArray :: operator [ ] ( int iIndex ){
	return GetAt(iIndex);
	}

CItem * CItemArray :: First( ){
	if( m_iItems>0 )
		return m_aItems[0];
	return NULL;
	}

CItem * CItemArray :: Last( ){
	if( m_iItems>0 )
		return m_aItems[m_iItems-1];
	return NULL;
	}

//-----------------------------------------------------------------------------
// return days since 1900 ( converted from seconds since 1970 )
//-----------------------------------------------------------------------------
double Now() { 
	__timeb64 tbuf;
	_ftime64( &tbuf );
	return 25569.0+double(tbuf.time-tbuf.timezone*60)/(60.0*60.0*24.0);
	}

CString CurrentTime(bool lSec, bool lSep) { 
	SYSTEMTIME st;
	GetLocalTime( &st );
	CString c("yyyy-mm-dd hh:mm:ss                ");
	if( lSec ) { 
		sprintf( c.Buffer(), lSep ? "%4d-%02d-%02d %02d:%02d:%02d" : "%4d%02d%02d%02d%02d%02d", 
			int(st.wYear), int(st.wMonth), int(st.wDay), 
			int(st.wHour), int(st.wMinute), int(st.wSecond) );
		}
	else { 
		sprintf( c.Buffer(), lSep ? "%4d-%02d-%02d %02d:%02d" : "%4d%02d%02d%02d%02d", 
			int(st.wYear), int(st.wMonth), int(st.wDay), 
			int(st.wHour), int(st.wMinute) );
		}	
	c.setlength();
	return c;
	}


//-----------------------------------------------------------------------------
//-- Window utilities
// Register a window class
void CreateWindowClass( const char * szName, WNDPROC pProc, HINSTANCE hInst, LPCTSTR iIcon, 
					LPCTSTR iIconSmall, int iExtraBytes, int iColor ) { 
	//----- register main window class
	WNDCLASSEX wcex;
	wcex.cbSize = sizeof(WNDCLASSEX); 
	wcex.style			= CS_HREDRAW | CS_VREDRAW ;
	wcex.lpfnWndProc	= (WNDPROC)pProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= iExtraBytes;
	wcex.hInstance		= hInst;
	wcex.hIcon			= LoadIcon( hInst, (LPCTSTR)iIcon);
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(iColor+1);		
	wcex.lpszMenuName	= NULL;
	wcex.lpszClassName	= szName;
	wcex.hIconSm		= LoadIcon(wcex.hInstance, (LPCTSTR)iIconSmall);
	RegisterClassEx(&wcex);
	}


//-----------------------------------------------------------------------------
CBMP :: CBMP( ) { 
	m_achBuf=NULL;
	}
	
//-- copy constructor with resizing
CBMP :: CBMP( const CBMP & ref, COLORREF crFill, int iWid, int iHgt ) { 
	//--- copy most params, 
	m_iPos = ref.m_iPos;
	m_iBytesPerPixel = ref.m_iBytesPerPixel;
	m_iBitsPerPixel = ref.m_iBitsPerPixel;

	//-- width and height if not -1 should be used to change bitmap size.
	//-- if width is changed, BytesPerRow should be recalculated
	m_iHgt = (iHgt==-1?ref.m_iHgt:iHgt);
	if( iWid==-1 ) {
		m_iWid = ref.m_iWid;
		m_iBytesPerRow = ref.m_iBytesPerRow;
		}
	else { 
		m_iWid = iWid;
		m_iBytesPerRow = m_iBytesPerPixel * m_iWid;
		m_iBytesPerRow += ((4-(m_iBytesPerRow%4))%4);
		}
		
	//-- copy pixels
	int iCol, iRow, iRowCopySize=m_iBytesPerPixel*min(m_iWid,ref.m_iWid);
	m_achBuf = new BYTE[ m_iBytesPerRow * m_iHgt ];
	for( iRow=0; iRow<m_iHgt && iRow<ref.m_iHgt; ++iRow ) {
		// bulk copy lesser of source and dest widths pixels 
		memcpy( m_achBuf+(m_iHgt-iRow-1)*m_iBytesPerRow, 
				ref.m_achBuf+(ref.m_iHgt-iRow-1)*ref.m_iBytesPerRow, 
				iRowCopySize );
		// individual fill background if dest is wider than source
		for( iCol=ref.m_iWid; iCol<m_iWid; ++iCol ) 		
			SetPixel( iCol, m_iHgt-iRow-1, crFill );
		}
	for( iRow=ref.m_iHgt; iRow<m_iHgt; ++iRow ) 
		for( iCol=0; iCol<m_iWid; ++iCol ) 		
			SetPixel( iCol, m_iHgt-iRow-1, crFill );
	}

CBMP :: ~CBMP() { 
	if( m_achBuf!=NULL ) 
		delete[] m_achBuf;
	}
		
BYTE * CBMP :: Pixel( int iRow, int iCol ) { 
	return m_achBuf+m_iBytesPerRow*iRow+iCol*m_iBytesPerPixel;
	}
		
bool CBMP :: Load( const char * pchFile ) { 
	//-- read bmp file header	
	FILE * h = fopen( pchFile, "rb");
	if( h == NULL ) 
		return false;
		
	fseek( h, 10, SEEK_SET );
	fread( &m_iPos, 4, 1, h );
	fseek( h, 18, SEEK_SET );
	fread( &m_iWid, 4, 1, h );
	fread( &m_iHgt, 4, 1, h );
	fseek( h, 28, SEEK_SET );
	fread( &m_iBitsPerPixel, 2, 1, h );
	m_iBytesPerPixel=m_iBitsPerPixel/8;
	if( m_iBytesPerPixel<3 || m_iBytesPerPixel>4 ) { 
		MessageBox( NULL, "Error reading bitmap file", pchFile, MB_OK );
		fclose(h);
		return false;
		}
	m_iBytesPerRow=m_iBytesPerPixel*m_iWid;
    m_iBytesPerRow += ((4-(m_iBytesPerRow%4))%4);
    m_iHgt=min(800,m_iHgt);			// limit height to 800 - SWF limitation ?

	//-- Read BMP file data
    fseek( h, m_iPos, SEEK_SET );
    m_achBuf = new BYTE[m_iHgt*m_iBytesPerRow];
    fread( m_achBuf, m_iHgt, m_iBytesPerRow, h );
	
    fclose(h);
    return true;
    }

bool CBMP :: Save( const char * pchFile ) { 
	//-- read bmp file header	
	FILE * h = fopen( pchFile, "wb");
	if( h == NULL ) 
		return false;
	
	char buf1[14] = { 'B', 'M' };		
	*(int *)(buf1+2) = 14 + 40 + m_iBytesPerRow * m_iHgt;		// file size 
	*(int *)(buf1+6) = 0;									// reserved - 0 
	*(int *)(buf1+10) = 54;									// offset to data 

	char buf2[40];
	*(int *)(buf2+0)=40;			// Header size in bytes      
	*(int *)(buf2+4)=m_iWid;
	*(int *)(buf2+8)=m_iHgt;		// Width and height of image 
	*(short *)(buf2+12)=1;		// Number of colour planes   
	*(short *)(buf2+14)=24;		// Bits per pixel            
	*(int *)(buf2+16)=0;			// Compression type          
	*(int *)(buf2+20)=m_iBytesPerRow * m_iHgt;	// Image size in bytes       
	*(int *)(buf2+22)=2799;
	*(int *)(buf2+28)=2799;		// Pixels per meter          
	*(int *)(buf2+32)=0;         // Number of colours         
	*(int *)(buf2+36)=0;			// Important colours         

	if( fwrite( buf1, 14, 1, h )!=1 
		|| fwrite( buf2, 40, 1, h )!=1
		|| fwrite( m_achBuf, m_iBytesPerRow, m_iHgt, h )!=m_iHgt ) {
		fclose(h);
		return false;
		}

	fclose(h);
	return true;
    }

void CBMP :: SetPixel( int iX, int iY, COLORREF cr ) { 
	BYTE * pMem = m_achBuf+m_iBytesPerRow*iY+iX*m_iBytesPerPixel;
	pMem[0]=GetBValue(cr);			// BLUE
	pMem[1]=GetGValue(cr);			// GREEN
	pMem[2]=GetRValue(cr);			// RED
	if( m_iBytesPerPixel==4 ) 
		pMem[3]=0;
	}
	
void CBMP :: HLine( int iX0, int iX1, int iY0, COLORREF cr ) { 
	if( iY0<0 || iY0>=m_iHgt ) 
		return;
	int iXX0 = max(0,min(iX0,iX1));
	int iXX1 = min(m_iWid-1,max(iX0,iX1));
	BYTE * pMem = m_achBuf+m_iBytesPerRow*iY0+iXX0*m_iBytesPerPixel;
	
	for( int iXX=iXX0; iXX<=iXX1; ++iXX ) {			// BYTE * pCR = (BYTE *)&cr;
		pMem[0]=GetBValue(cr);						// BLUE		//pCR[2];
		pMem[1]=GetGValue(cr);						// GREEN	//pCR[1];
		pMem[2]=GetRValue(cr);						// RED		//pCR[0];
		if( m_iBytesPerPixel==4 ) 
			pMem[3]=0;
		pMem+=m_iBytesPerPixel;
		}
	}
	
void CBMP :: VLine( int iX0, int iY0, int iY1, COLORREF cr ) { 
	if( iX0<0 || iX0>=m_iWid ) 
		return;
	int iYY0 = max(0,min(iY0,iY1));
	int iYY1 = min(m_iHgt-1,max(iY0,iY1));
	BYTE * pMem = m_achBuf+m_iBytesPerRow*iYY0+iX0*m_iBytesPerPixel;
	for( int iYY=iYY0; iYY<=iYY1; ++iYY ) { 
		pMem[0]=GetBValue(cr);			// BLUE
		pMem[1]=GetGValue(cr);			// GREEN
		pMem[2]=GetRValue(cr);			// RED
		if( m_iBytesPerPixel==4 ) 
			pMem[3]=0;
		pMem+=m_iBytesPerRow;
		}
	}
	
void CBMP :: FillRect( int iX0, int iY0, int iX1, int iY1, COLORREF cr ) { 
	for( int iY=min(iY0,iY1); iY<max(iY0,iY1); ++iY ) 
		HLine( iX0, iX1, iY, cr );
	}
	
void CBMP :: FrameRect( int iX0, int iY0, int iX1, int iY1, COLORREF cr ) { 
	HLine( iX0, iX1, iY0, cr );
	HLine( iX0, iX1, iY1, cr );
	VLine( iX0, iY0, iY1, cr );
	VLine( iX1, iY0, iY1, cr );
	}

//-----------------------------------------------------------------------------
CWAV :: CWAV() { 
	m_achBuf=NULL;
	}
	
CWAV :: ~CWAV() { 
	if( m_achBuf!=NULL ) 
		delete[] m_achBuf;
	}

#ifdef _DEBUG
void LogArray_d( FILE * h, const char * pchName, double * pd, int iLen )  { 
	fprintf( h, "%s=", pchName );
	for( int i=0; i<iLen; ++i ) 
		fprintf( h, "%c %.5f", i==0?'{':',', pd[i] );
	fprintf( h, "}\n");
	}
	
void LogArray_dd( FILE * h, const char * pchName, double ** pd, int iRows, int iCols )  { 
	fprintf( h, "%s=\n", pchName );
	for( int iRow=0; iRow<iRows; ++iRow ) {
		fprintf( h, "\t%d\t", iRow );
		for( int iCol=0; iCol<iCols; ++iCol )
			fprintf( h, "%c %.5f", iCol==0?'{':',', pd[iRow][iCol] );
		fprintf( h, "}\n");
		}
	}
	
void LogArray_ddd( FILE * h, const char * pchName, double *** pd, int iM, int iRows, int iCols ) { 
	fprintf( h, "%s[%d][%d][%d]=\n", pchName, iM, iRows, iCols );
	
	for( int i=0; i<iM; ++i ) {
		fprintf( h, "\t[%d]\t{ ", i );
		for( int iRow=0; iRow<iRows; ++iRow ) {
			for( int iCol=0; iCol<iCols; ++iCol )
				fprintf( h, "%c %.5f", iCol==0?'{':',', pd[i][iRow][iCol] );
			fprintf( h, "}");
			}
		fprintf( h, "}\n");
		}
	}

void LogArray_f( FILE * h, const char * pchName, float * pf, int iLen )  { 
	fprintf( h, "%s=", pchName );
	for( int i=0; i<iLen; ++i ) 
		fprintf( h, "%c %.5f", i==0?'{':',', double(pf[i]) );
	fprintf( h, "}\n");
	}

void LogArray_i( FILE * h, const char * pchName, int * pi, int iLen )  { 
	fprintf( h, "%s=", pchName );
	for( int i=0; i<iLen; ++i ) 
		fprintf( h, "%c %d", i==0?'{':',', pi[i] );
	fprintf( h, "}\n");
	}
#endif

double RandomD( double dMin, double dMax, int iType ) { 
	const double LNRNDMX = log( double( RAND_MAX+1 ) );

	if( iType==1 )	// exponentially distributed random 
		return dMin + (log(double(rand()+1))) * (dMax-dMin) / LNRNDMX;
		
	
	if( iType==-1 )	// inverse exponentially distributed random 
		return dMin + (LNRNDMX - log(double(rand()+1))) * (dMax-dMin) / LNRNDMX;
	
	// linearly distributed
	return dMin + double(rand()) * (dMax-dMin) / RAND_MAX;
	}

//-----------------------------------------------------------------------------------
void CItemArray :: SortSwap(int iLow,int iHigh){ 
	CItem * p1;
	p1 = GetAt(iLow);
	m_aItems[iLow] = GetAt(iHigh);
	m_aItems[iHigh] = p1;
	}

void CItemArray :: SubSort( int iLowInit, int iHighInit, CString(* KeyValue)(CItem * pRow) ){
	int	iLow=iLowInit, iHigh=iHighInit;
	CItem * p1;
	
	//----- check if sort is trivial
	if( iLow>=iHigh )
		return;
	
	if( iLow==iHigh-1 ){
		if( !(KeyValue(GetAt(iLow))<=KeyValue(GetAt(iHigh))) )
			SortSwap(iLow,iHigh);
		return;
		}
	
	//----- pick pivot from middle of list 
	if( iHigh-iLow>2 )
		SortSwap( iLow, int((iLow+iHigh)/2) );
	
	p1 = GetAt(iLow);
	++iLow;
	
	//----- Move all <= pivot prior to all > pivot
	while( iLow<iHigh ){
		while( iLow<=iHighInit && KeyValue(GetAt(iLow))<=KeyValue(p1) )
			++iLow;
		while( iHigh-1>iLowInit && KeyValue(p1)<=KeyValue(GetAt(iHigh)) )
			--iHigh;
		if( iLow<iHigh )
			SortSwap( iLow, iHigh );
		}
	
	//----- Sub sort on two halves 
	if( iLow-1 > iLowInit ) 
		SortSwap( iLowInit, iLow-1 );
	SubSort( iLowInit, iLow-2, KeyValue );
	if( iLow<iHighInit )
		SubSort( iLow, iHighInit, KeyValue );
	
	return;
	}

void CItemArray :: Sort(CString(* KeyValue)(CItem * pRow)){ 
	SubSort( 0, Count()-1, KeyValue );
	}

	